@@ -39,6 +39,7 @@ sub init {
     $self->{sqla} = Nitesi::Query::DBI->new(%q_args);
 
     hook 'after_cart_add' => sub {$self->_after_cart_add(@_)};
+    hook 'after_cart_update' => sub {$self->_after_cart_update(@_)};
     hook 'after_cart_remove' => sub {$self->_after_cart_remove(@_)};
     hook 'after_cart_rename' => sub {$self->_after_cart_rename(@_)};
     hook 'after_cart_clear' => sub {$self->_after_cart_clear(@_)};
@@ -52,7 +53,7 @@ Loads cart from database.
 
 sub load {
     my ($self, %args) = @_;
-    my ($uid, $result, $code, %specs);
+    my ($uid, $name, $code);
 
     # check whether user is authenticated or not
     unless ($uid = $args{uid} || 0) {
@@ -70,17 +71,32 @@ sub load {
     }
     $self->{id} = $code;
 
-    # build query for item retrieval
-    %specs = (fields => $self->{settings}->{fields} || 
-	      [qw/products.sku products.name cart_products.quantity/],
-	      join => $self->{settings}->{join} ||
-	      [qw/carts code=cart cart_products sku=sku products/],
-	      where => {'carts.name' => $self->name, uid => $uid});	      
+    $self->_load_cart;    
+}
 
-    # retrieve items from database
-    $result = $self->{sqla}->select(%specs);
+=head2 id
 
-    $self->seed($result);
+Return cart identifier.
+
+=cut
+
+sub id {
+    my $self = shift;
+
+    if (@_ && defined ($_[0])) {
+        my $id = $_[0];
+
+        if ($id =~ /^[0-9]+$/) {
+            $self->{id} = $id;
+            $self->_load_cart;
+        }
+    }
+    elsif (! $self->{id}) {
+        # forces us to create entry in cart table
+        $self->_create_cart;
+    }
+
+    return $self->{id};
 }
 
 =head2 save
@@ -93,6 +109,33 @@ sub save {
     return 1;
 }
 
+# creates cart in database
+sub _create_cart {
+    my $self = shift;
+
+	$self->{id} = $self->{sqla}->insert('carts', {name => $self->name,
+                                                  uid => $self->{uid}});
+}
+
+# loads cart from database
+sub _load_cart {
+    my $self = shift;
+
+    # build query for item retrieval
+    my %specs = (fields => $self->{settings}->{fields} || 
+                 [qw/products.sku products.name cart_products.quantity/],
+                 join => $self->{settings}->{join} ||
+                 [qw/carts code=cart cart_products sku=sku products/],
+                 where => {'carts.code' => $self->{id}},
+                 );
+
+    # retrieve items from database
+    my $result = $self->{sqla}->select(%specs);
+
+    $self->seed($result);
+}
+
+# hook methods
 sub _after_cart_add {
     my ($self, @args) = @_;
     my ($item, $update, $record);
@@ -105,28 +148,44 @@ sub _after_cart_add {
     $item = $args[1];
     $update = $args[2];
 
-    unless ($self->{code}) {
-	# need to create cart first
-	$self->{sqla}->insert('carts', {name => $self->name,
-					uid => $self->{uid}});
-  
-	# determine cart code
-	$self->{code} = $self->{sqla}->select_field(table => 'carts', field => 'code', 
-						    where => {name => $self->name, uid => $self->{uid}});
+    unless ($self->{id}) {
+        $self->_create_cart;
     }
 
     if ($update) {
 	# update item in database
 	$record = {quantity => $item->{quantity}};
-	$self->{sqla}->update('cart_products', $record, {cart => $self->{code}, sku => $item->{sku}});
+	$self->{sqla}->update('cart_products', $record, {cart => $self->{id}, sku => $item->{sku}});
     }
     else {
 	# add new item to database
-	$record = {cart => $self->{code}, sku => $item->{sku}, quantity => $item->{quantity}, position => 0};
+	$record = {cart => $self->{id}, sku => $item->{sku}, quantity => $item->{quantity}, position => 0};
 	$self->{sqla}->insert('cart_products', $record);
     }
 }
 
+sub _after_cart_update {
+    my ($self, @args) = @_;
+    my ($item, $new_item, $count);
+
+    unless ($self eq $args[0]) {
+	# not our cart
+	return;
+    }
+
+    $item = $args[1];
+    $new_item = $args[2];
+
+    # update item in database
+    Dancer::Logger::debug("Updating cart products with: ", $new_item);
+
+    $count = $self->{sqla}->update(table => 'cart_products', 
+				   set => $new_item, 
+				   where => {cart => $self->{id}, sku => $item->{sku}});
+
+    Dancer::Logger::debug("Items updated: $count.");
+}
+
 sub _after_cart_remove {
     my ($self, @args) = @_;
     my ($item);
